// AppOctetStream.cpp: implementation of the CAppOctetStream class.
// Author: Wes Clyburn (clyburnw@enmu.edu)
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "longstr.h"
#include "AppOctetStream.h"
#include "Base64.h"
#include "MIMEMessage.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

// IMPORTANT: The number of bytes we read must be
//  a multiple of 3 because CBase64's Encode()
//  method will append padding characters ('=')
//  to make the output's size a multiple of 4.
//  (Base64 treats 3 8-bit bytes as 4 6-bit 'bytes').
//  MIME decoders are free to treat '=' as a signal
//  that there's no more data, so we don't want to pad
//  until we're supposed to.
// When at the end of the file, the # of bytes read
//  may not be a multiple of 3, but that's okay
//  because we DO want the padding chars then.

#define BYTES_TO_READ 54 // This number guarantess output won't
						 // won't exceed line-length limit

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CAppOctetStream::CAppOctetStream( int nContentType)
	:CMIMEContentAgent( nContentType)
{
}

CAppOctetStream::~CAppOctetStream()
{
}


BOOL CAppOctetStream::AppendPart( CMIMEMessage::CMIMEPart* pMIMEPart ,CMIMEMessage *pMIMEMessage )
{
	ASSERT(!pMIMEPart->m_sContent.IsEmpty());
	int nEncoding = pMIMEPart->m_nEncoding;
	m_pMIMEMessage=pMIMEMessage;
	ASSERT_VALID(m_pMIMEMessage);
	pMIMEMessage->m_sBody += build_sub_header( pMIMEPart);
	if( (nEncoding & CMIMEMessage::MEMORY) != 0) // default is FILE
	{
		attach_memory(pMIMEPart->m_sContent, nEncoding, pMIMEMessage->m_sBody );
	}
	else
	{
		CStdioFile fAttachment;
		if( !fAttachment.Open( pMIMEPart->m_sContent, (CFile::modeRead | CFile::shareDenyWrite | CFile::typeBinary) ) )
			return FALSE;
		attach_file( &fAttachment, nEncoding, pMIMEMessage->m_sBody );
		fAttachment.Close();
	}
	return TRUE;
}

CString CAppOctetStream::build_sub_header(CMIMEMessage::CMIMEPart* pMIMEPart)
{
	CString sSubHeader;
	CString sTemp;
	int nEncoding = pMIMEPart->m_nEncoding;
	ASSERT( ((nEncoding & 0x0ffff) ==0) || ((nEncoding & 0x0ffff) == CMIMEMessage::BASE64));
	if (m_nMIMETypeIHandle == CMIMEMessage::APPLICATION_OCTETSTREAM_IMAGE)
	{
		sSubHeader.Format( _T( "Content-Type: %s\r\n" ),
							(LPCTSTR)GetContentTypeString());
		sSubHeader += _T( "Content-Transfer-Encoding: base64\r\n" );
		sTemp.Format( _T( "Content-ID: <%s>\r\n" ),pMIMEPart->m_sParameters);
	}
	else 
	{		// APPLICATION_OCTETSTREAM  Attachment
		ASSERT (m_nMIMETypeIHandle == CMIMEMessage::APPLICATION_OCTETSTREAM);
		TCHAR szFName[_MAX_FNAME];
		TCHAR szExt[_MAX_EXT];
		if( (nEncoding & CMIMEMessage::MEMORY) != 0)
		{
			sTemp = pMIMEPart->m_sParameters; // file nmae is in parameters in this case
			ASSERT(!sTemp.IsEmpty());
		}
		else
			sTemp = pMIMEPart->m_sContent;
		_tsplitpath( sTemp, NULL, NULL, szFName, szExt );
		if( nEncoding & CMIMEMessage::BPATH )
			sTemp.Format( "; file=%s%s", szFName, szExt );
		else
			sTemp = _T( "" );
		sSubHeader.Format( _T( "Content-Type: %s%s\r\n" ),
							(LPCTSTR)GetContentTypeString(),
							(LPCTSTR)sTemp );
		sSubHeader += _T( "Content-Transfer-Encoding: base64\r\n" );
		sTemp.Format( _T( "Content-Disposition: attachment; filename=%s%s\r\n" ),
					  szFName, szExt );
	}
	sSubHeader += sTemp;
	// Signal end of sub-header.
	sSubHeader += _T( "\r\n" ); // Warning: numerous concatenations
								// are inefficient.
	return sSubHeader;


}



CString CAppOctetStream::GetContentTypeString()
{
	CString s;
	s = _T( "application/octet-stream" );
	return s;
}

// Caller is responsible for opening and closing the file
void CAppOctetStream::attach_file(CStdioFile* pFileAtt, 
								  int nEncoding, 
								  CLongString & sDestination/*,CMIMEMessage pMIMEMessage*/)
{
	CMIMECode* pEncoder;
	int nBytesRead;
	int TotalSizeRead;
	int TotalSizeDisplayed=0;
	TotalSizeRead=0;
	TotalSizeDisplayed=0;
	TCHAR szBuffer[ BYTES_TO_READ + 1 ];

	ASSERT( pFileAtt != NULL );
	if( pFileAtt == NULL )
		return;
	ASSERT( (nEncoding & CMIMEMessage::SOONCODED)==0); 
//      If you fall in this assert, you have tried to send soon coded data.
//		Even if it seems trivial, it is not soon implemented
//		If you decide to implement it send me the result at pcouderc@tol.Fr
	switch( nEncoding & 0x0ffff )
	{
		// This class handles only Base64 encoding, but others
		//  may be added here.
		default:
					// Only Base 64
			// Fall through to...
		case CMIMEMessage::BASE64:
			try 
			{
				pEncoder = new CBase64;
			}
			catch( CMemoryException* e )
			{
				delete e;
				return;
			}
	}
	if( pEncoder == NULL )	// Old habits are hard to break
		return;
	sDestination.OptimizeGrowing(300000);
	do
	{
		try
		{
			nBytesRead = pFileAtt->Read( szBuffer, BYTES_TO_READ );
			TotalSizeRead+=nBytesRead;
			int Display=TotalSizeRead/10000;
			if(Display!=TotalSizeDisplayed)
			{
				TotalSizeDisplayed=Display;
				TRACE("--- %dK\n",TotalSizeDisplayed*10);
				if(m_pMIMEMessage!=NULL)
				{
					ASSERT_VALID(m_pMIMEMessage);
					m_pMIMEMessage->NotifyProgress(TotalSizeDisplayed*10,NULL);
				}
			}
		}
		catch( CFileException* e )
		{
			delete e;
			break;
		}
		szBuffer[ nBytesRead ] = 0;	// Terminate the string
		sDestination += pEncoder->Encode( szBuffer, nBytesRead );
		sDestination += _T( "\r\n" );
	} while( nBytesRead == BYTES_TO_READ );
	sDestination += _T( "\r\n" );
	sDestination.EndOptimizeGrowing();
	delete pEncoder;
}



void CAppOctetStream::attach_memory(CString& sContent, int nEncoding, CLongString & sDestination)
{
	CMIMECode* pEncoder;
	int nBytesRead;
	int TotalSizeRead;
	int TotalSizeDisplayed=0;
	TotalSizeRead=0;
	TotalSizeDisplayed=0;

	ASSERT( (nEncoding & CMIMEMessage::SOONCODED)==0); 
//      If you fall in this assert, you have tried to send soon coded data.
//		Even if it seems trivial, it is not soon implemented
//		If you decide to implementit send me the result at pcouderc@tol.Fr
	switch( nEncoding & 0x0ffff )
	{
		// This class handles only Base64 encoding, but others
		//  may be added here.
		default:
					// Only Base 64
			// Fall through to...
		case CMIMEMessage::BASE64:
			try 
			{
				pEncoder = new CBase64;
			}
			catch( CMemoryException* e )
			{
				delete e;
				return;
			}
	}
	if( pEncoder == NULL )	// Old habits are hard to break
		return;
	sDestination.OptimizeGrowing(300000);
	int StringLength = sContent.GetLength(); 
	LPTSTR pBuffer = sContent.GetBuffer(StringLength);
	do
	{
		try
		{
			nBytesRead = StringLength-TotalSizeRead;
			if(nBytesRead > BYTES_TO_READ) nBytesRead = BYTES_TO_READ;
			TotalSizeRead+=nBytesRead;
			int Display=TotalSizeRead/10000;
			if(Display!=TotalSizeDisplayed)
			{
				TotalSizeDisplayed=Display;
				TRACE("--- %dK\n",TotalSizeDisplayed*10);
				if(m_pMIMEMessage!=NULL)
				{
					ASSERT_VALID(m_pMIMEMessage);
					m_pMIMEMessage->NotifyProgress(TotalSizeDisplayed*10,NULL);
				}
			}
		}
		catch( CFileException* e )
		{
			delete e;
			break;
		}
// 		pBuffer[ nBytesRead ] = 0;	// Terminate the string ???? set in comment by PC
		sDestination += pEncoder->Encode( pBuffer, nBytesRead );
		sDestination += _T( "\r\n" );
		pBuffer += nBytesRead;
	} while( TotalSizeRead != StringLength );
	sDestination += _T( "\r\n" );
	sDestination.EndOptimizeGrowing();
	delete pEncoder;
}
