// MIMEMessage.cpp: implementation of the CMIMEMessage class.
// Author: Wes Clyburn (clyburnw@enmu.edu)
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "longstr.h"
#include "MIMEMessage.h"
#include "TextPlain.h"
#include "AppOctetStream.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif



class CMIMEContainer : public CMIMEContentAgent  
{
public:
	int m_nContentType;

	CMIMEContainer( int nContentType, UINT nWrapPos = 72);
	virtual ~CMIMEContainer();

	virtual BOOL AppendPart( LPCTSTR szContent, LPCTSTR szParameters, int nEncoding,  CLongString& sDestination,CMIMEMessage *pMIMEMessage  );
	virtual CString GetContentTypeString();
	BOOL AppendPart( CMIMEMessage::CMIMEPart* pMIMEPart ,CMIMEMessage *pMIMEMessage );

protected:
	UINT m_nWrapPos;

	CString wrap_text( LPCTSTR szText );
	virtual CString build_sub_header( LPCTSTR szContent, LPCTSTR szParameters, int nEncoding);
	void insert_boundary( CString& sText );
    void insert_message_end( CString& sText );
	static LPSTR m_pPartBoundary;
	static char m_BoundaryLevel;
};




// Static Member Initializers
CMIMEMessage::CMIMETypeManager CMIMEMessage::m_MIMETypeManager;

//     WARNING : this 0 must not be changed :
LPSTR CMIMEContainer::m_pPartBoundary=_T( "_.__.__TOL__Mailer__Part_Boundary_" );
char CMIMEContainer::m_BoundaryLevel ='0';
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CMIMEMessage::CMIMEMessage()
{
	 

	// Register the MIME types handled by this class
	//
	CMIMEContentAgent* pType;
	// These objects are deleted by CMIMTypeManager's destructor
	pType = new CTextPlain( TEXT_PLAIN, GetCharsPerLine() );
	register_mime_type( pType );
	pType = new CAppOctetStream( APPLICATION_OCTETSTREAM);
	register_mime_type( pType );
	pType = new CAppOctetStream( APPLICATION_OCTETSTREAM_IMAGE);
	register_mime_type( pType );
	pType = new CTextPlain( TEXT_HTML, GetCharsPerLine() );
	register_mime_type( pType );
	pType = new CMIMEContainer( MIXED, GetCharsPerLine() );
	register_mime_type( pType );
	pType = new CMIMEContainer( RELATED, GetCharsPerLine() );
	register_mime_type( pType );
	pType = new CMIMEContainer( ALTERNATIVE, GetCharsPerLine() );
	register_mime_type( pType );
	m_pMainMIMEPart = NULL;
	m_FirstMultipart = true;
}

CMIMEMessage::~CMIMEMessage()
{
	if(m_pMainMIMEPart!=NULL) delete m_pMainMIMEPart;
}


CMIMEMessage::CMIMEPart::CMIMEPart(LPCTSTR szContent,
							   int nContentType, 
							   LPCTSTR szParameters, 
							   int nEncoding, 
							   int Len )
{
	m_nContentType = nContentType;
	if(szParameters != NULL) m_sParameters = szParameters;
	m_nEncoding = nEncoding;
	m_Len = Len;
	if(szContent != NULL)
	{
		if(Len == 0)
		{
			m_sContent.TrimLeft();
			m_sContent.TrimRight();
			m_sContent = szContent;
		}
		else
		{
			LPTSTR p= m_sContent.GetBufferSetLength(Len);
			memcpy(p, szContent, Len);
			m_sContent.ReleaseBuffer(Len);
		}
	}
}


CMIMEMessage::CMIMEPart* CMIMEMessage::CMIMEPart::AddMIMEPart(eMIMETypeCode nContentType, 
							   LPCTSTR szContent,
							   eMIMEEncodingCode nEncoding ,
							   LPCTSTR szParameters , 
							   int Len)
{
	CMIMEPart *pPart= new CMIMEPart( szContent,
							   nContentType, szParameters, nEncoding, Len );
	m_MIMEPartList.AddTail( pPart );
	return pPart;
}



void CMIMEMessage::CMIMEPart::append_mime_parts(CMIMEMessage* pMIMEMessage)
{
	CMIMEContentAgent* pMIMEType;
	pMIMEType = pMIMEMessage->m_MIMETypeManager.GetHandler( m_nContentType );
	ASSERT( pMIMEType != NULL );
	if( pMIMEType != NULL )
	{
		pMIMEMessage->NotifyProgress(1,m_sContent);
		pMIMEType->AppendPart(this, pMIMEMessage);
	}

}



void CMIMEMessage::prepare_header()
{
	CString sTemp;

	// Let the base class add its headers
	CMailMessage::prepare_header();
	
	add_header_line( _T( "MIME-Version: 1.0" ) );
/*	sTemp.Format( _T( "Content-Type: %s; boundary=%s" ),
				  (LPCTSTR)m_sMIMEContentType,
				  (LPCTSTR)m_sPartBoundary );
	add_header_line( (LPCTSTR)sTemp );
*/
}



// This implementation adds the part to the part-list used
//  to build the body.
CMIMEMessage::CMIMEPart*  CMIMEMessage::AddMIMEPart(LPCTSTR szContent,
							   int nContentType, 
							   LPCTSTR szParameters, 
							   int nEncoding, 
							   BOOL bPath )
{
	CMIMEPart *pPart= new CMIMEPart( szContent,
							   nContentType, szParameters, nEncoding | (bPath ? BPATH : 0));
	if(m_pMainMIMEPart == NULL)
	{
		if( nContentType<MIXED) // ForCompatibility with previous version
			m_pMainMIMEPart = new CMIMEPart("",MIXED,"",0,0);
		else
		{
//			new normal use
			m_pMainMIMEPart = pPart;
			return pPart;
		}
	}
	if( nContentType == TEXT_PLAIN )
		m_pMainMIMEPart->m_MIMEPartList.AddHead( pPart );
	else
		m_pMainMIMEPart->m_MIMEPartList.AddTail( pPart );
	return pPart;
}


CMIMEMessage::CMIMEPart*  CMIMEMessage::AddMIMEPart(eMIMETypeCode nContentType)
{
	CMIMEPart *pPart= new CMIMEPart( NULL, nContentType, 0, NULL);
	m_pMainMIMEPart = pPart;
	return pPart;
}





void CheckLongString(int Len,LPCSTR pBuff,LPCSTR cmt=NULL);

void CMIMEMessage::prepare_body()
{
	// Class user may have assigned body text directly.
	// Convert it to just another MIME part to be processed.
	// If this default Content-Type isn't good enough for the
	// class user, he or she should have used AddMIMEPart() instead.
	if( m_sBody != _T( "" ) )
	{
		if(	m_pMainMIMEPart == NULL)
			m_pMainMIMEPart = new CMIMEPart((LPCTSTR)m_sBody, TEXT_PLAIN, "", _7BIT, FALSE );
		else
			AddMIMEPart( (LPCTSTR)m_sBody, TEXT_PLAIN, "", _7BIT, FALSE );
	}

	// Initialize the body (replace current contents).
	m_sBody.Empty();
	m_pMainMIMEPart->append_mime_parts(this);
	// Let the base class take me to Funky Town
	CMailMessage::prepare_body();

}

void CMIMEMessage::register_mime_type(CMIMEContentAgent* pMIMEType)
{
	ASSERT( pMIMEType != NULL );
	if( pMIMEType == NULL )
		return;
	m_MIMETypeManager.RegisterMIMEType( pMIMEType );
}




//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
// CMIMEContainer Implementation
//////////////////////////////////////////////////////////////////////

CMIMEContainer::CMIMEContainer( int nContentType, UINT nWrapPos )
	:CMIMEContentAgent( nContentType)
{
	m_nWrapPos = nWrapPos;
	m_nContentType = nContentType;
}

CMIMEContainer::~CMIMEContainer()
{

}

CString CMIMEContainer::GetContentTypeString()
{
	CString s;
	switch (m_nContentType)
	{
	case CMIMEMessage::MIXED: s = _T( "multipart/mixed" );
		break;
	case CMIMEMessage::ALTERNATIVE: s = _T( "multipart/alternative" );
		break;
	case CMIMEMessage::RELATED: s = _T( "multipart/related" );
		break;
	default: s = _T( "multipart/UNKNOWN" );
		ASSERT(FALSE);
	}
	return s;
}

BOOL CMIMEContainer::AppendPart(LPCTSTR szContent, 
							LPCTSTR szParameters, 
							int nEncoding, 
							CLongString & sDestination,
							CMIMEMessage *pMIMEMessage )
{
	ASSERT(FALSE);
	return FALSE;
}

void CMIMEContainer::insert_boundary( CString& sText )
{
	CString sTemp;

	if( sText.Right( 2 ) != _T( "\r\n" ) )
		sText += _T( "\r\n" );
sText += _T( "\r\n" );
	sTemp.Format( _T( "--_%c%s\r\n" ),m_BoundaryLevel,(LPCTSTR)m_pPartBoundary );
	sText += sTemp;
}

void CMIMEContainer::insert_message_end( CString& sText )
{
	CString sTemp;
	if( sText.Right( 2 ) != _T( "\r\n" ) )
		sText += _T( "\r\n" );
	sTemp.Format( _T( "--_%c%s--\r\n" ),m_BoundaryLevel, (LPCTSTR)m_pPartBoundary );
	sText += sTemp;
}


BOOL CMIMEContainer::AppendPart( CMIMEMessage::CMIMEPart* pMIMEPart ,CMIMEMessage *pMIMEMessage )
{
	POSITION part_position;
	CMIMEMessage::CMIMEPart* pMIMESubPart = NULL;
	CMIMEContentAgent* pMIMEType = NULL;
	part_position = pMIMEPart->m_MIMEPartList.GetHeadPosition();
	// Get each part from the list, retrieve a handler for it,
	//  and let the handler do its thing.
	CString sSubHeader;
	sSubHeader = build_sub_header( pMIMEPart->m_sContent,
								   pMIMEPart->m_sParameters,
								   pMIMEPart->m_nEncoding),

	pMIMEMessage->m_sBody += (sSubHeader);
	if(	pMIMEMessage->m_FirstMultipart ) 
		{
			(pMIMEMessage->m_sBody) += _T( "This is a multi-part message in MIME format." );
			(pMIMEMessage->m_sBody) += _T( "\r\n\r\n" );
			pMIMEMessage->m_FirstMultipart = false;
		}

	while( part_position != NULL )
	{
		pMIMESubPart = pMIMEPart->m_MIMEPartList.GetNext( part_position );
		pMIMEType = pMIMEMessage->m_MIMETypeManager.GetHandler( pMIMESubPart->m_nContentType );
		if( pMIMEType != NULL )
		{
			insert_boundary( pMIMEMessage->m_sBody );
			pMIMEMessage->NotifyProgress(1,pMIMEPart->m_sContent);
			m_BoundaryLevel++; // Change boundary string in sub parts
			pMIMEType->AppendPart(pMIMESubPart ,pMIMEMessage);
			m_BoundaryLevel--;
		}
		delete pMIMESubPart;
	}
	insert_message_end( pMIMEMessage->m_sBody );
	return TRUE;
}


CString CMIMEContainer::build_sub_header(LPCTSTR szContent, 
									 LPCTSTR szParameters, 
									 int nEncoding) 
{
	CString sSubHeader;
	sSubHeader.Format( _T( "Content-Type: %s;%s\r\n\tboundary=_%c%s" ),
					   (LPCTSTR)GetContentTypeString(),
					   szParameters,m_BoundaryLevel,(LPCTSTR)m_pPartBoundary );
	sSubHeader += _T( "\r\n\r\n" );
	return sSubHeader;
}

CString CMIMEContainer::wrap_text(LPCTSTR szText)
{
	CString sTemp;
	CString sLeft;
	CString sRight;
	int lp = 0;
	UINT nCount = 0;
	int nSpacePos = 0;

	ASSERT( szText != NULL );
	if( szText == NULL )
		return sTemp;
	sTemp = szText;
	while( lp < sTemp.GetLength() )
	{
#ifdef _DEBUG
//           an ASSERT here detects a '\n' without '\r'
				if(sTemp[ lp ] == '\n') ASSERT(sTemp[ lp-1 ] == '\r');
#endif
		if( sTemp[ lp ] == ' ' )
			nSpacePos = lp;
		// Reset counter on newline
		if( sTemp.Mid( lp, 2 ) == _T( "\r\n" ) )
			nCount = 0;
		// Wrap text at last found space
		if( nCount > m_nWrapPos )
		{
			sLeft = sTemp.Left( nSpacePos );
			sRight = sTemp.Right( sTemp.GetLength() - nSpacePos );
			sLeft.TrimRight();
			sRight.TrimLeft();
			sLeft += _T( "\r\n" );
			sTemp = sLeft + sRight;
			nCount = 0;
		}
		else
			nCount++;
		lp++;
	}
	return sTemp;
}


//////////////////////////////////////////////////////////////////////
// CMIMETypeManager Implementation
//////////////////////////////////////////////////////////////////////

CMIMEMessage::CMIMETypeManager::CMIMETypeManager()
{
}



CMIMEMessage::CMIMETypeManager::~CMIMETypeManager()
{
	POSITION pos;
	CMIMEContentAgent* p;
	m_csAccess.Lock();
	pos = m_MIMETypeList.GetHeadPosition();
	while( pos != NULL )
	{
		p = m_MIMETypeList.GetNext( pos );
		delete p;
	}
}


void CMIMEMessage::CMIMETypeManager::RegisterMIMEType(CMIMEContentAgent *pMIMEType)
{
	ASSERT( pMIMEType != NULL );
	if( pMIMEType == NULL )
		return;
	m_csAccess.Lock();
	m_MIMETypeList.AddTail( pMIMEType );
}



CMIMEContentAgent* CMIMEMessage::CMIMETypeManager::GetHandler(int nContentType)
{
	POSITION pos;
	CMIMEContentAgent* pType = NULL;

	m_csAccess.Lock();
	pos = m_MIMETypeList.GetHeadPosition();
	while( pos != NULL )
	{
		pType = m_MIMETypeList.GetNext( pos );
		if( pType->QueryType( nContentType ) == TRUE )
			break;
	}
	ASSERT( pType->QueryType( nContentType ) == TRUE ); // forget to register a new CMIMEContentAgent?
	return pType;
}

