// MIMEMessage.h: interface for the CMIMEMessage class.
// Author: Wes Clyburn (clyburnw@enmu.edu) updated by Pierre Couderc
//         
//////////////////////////////////////////////////////////////////////


#if !defined(AFX_MIMEMESSAGE_H__6233D556_EBBF_11D1_870E_444553540001__INCLUDED_)
#define AFX_MIMEMESSAGE_H__6233D556_EBBF_11D1_870E_444553540001__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include <afxmt.h>
#include "MailMessage.h"

// CMIMEMessage
// Formats a message using the MIME specification.
// 

class CMIMEContentAgent;

class CMIMEMessage : public CMailMessage  
{
public:
	CMIMEMessage();
	virtual ~CMIMEMessage();
	class CMIMEPart;
	// MIME Type Codes
	enum eMIMETypeCode
	{
		TEXT_PLAIN = 0,
		APPLICATION_OCTETSTREAM,
		APPLICATION_OCTETSTREAM_IMAGE,
		TEXT_HTML,
//			Only containers from here
		MIXED=1000,
		ALTERNATIVE,
		RELATED
	};
	enum eMIMEEncodingCode
	{
//				Default values depend on current eMIMETypeCode 
		DEFAULT = 0,
		_7BIT = 1,
		_8BIT = 2,
		BINARY = 4,
		QUOTED_PRINTABLE = 0x10,
		BASE64 = 0x20,
		NEXT_FREE_ENCODING_CODE =0x40,
//							
		FILE =0x10000,  // Data source is in a file
		MEMORY =0x20000, // Data source is in memory
		SOONCODED =0x40000, // Data source is soon encoded
		ENCODE =0x80000, // Data source needs to be encoded
		BPATH =0x100000, // Include path for file attachmet
	};
protected:
	void register_mime_type( CMIMEContentAgent* pMIMEType );

	virtual void prepare_header();
	virtual void prepare_body();
	virtual void end_header() {;}

public:
	class CMIMEPart
	{
	public:
		//Data
		int m_nEncoding;
		int m_nContentType;
		CString m_sParameters;
		CString m_sContent;		
		CString m_ContentID;
		int m_Len;
		CList <CMIMEPart*, CMIMEPart*> m_MIMEPartList;
//		Methods
		CMIMEPart(LPCTSTR szContent,
							   int nContentType, 
							   LPCTSTR szParameters, 
							   int nEncoding, 
							   int Len=0 );
		CMIMEPart* AddMIMEPart(eMIMETypeCode nContentType, 
							   LPCTSTR szContent= NULL,
							   eMIMEEncodingCode nEncoding = DEFAULT,
							   LPCTSTR szParameters = NULL, 
							   int Len=0);
		void SetContentID(LPCTSTR contentID) { m_ContentID= contentID;}
	private:
		void append_mime_parts(CMIMEMessage* pMess);
		friend class CMIMEMessage;
//		friend class CMIMETypeManager;

	};
	friend class CMIMEPart;
	CMIMEPart* AddMIMEPart( LPCTSTR szContent,
					  int nContentType = APPLICATION_OCTETSTREAM,
					  LPCTSTR szParameters = _T( "" ), 
					  int nEncoding = BASE64, 
					  BOOL bPath = TRUE );
	CMIMEPart* AddMIMEPart(eMIMETypeCode nContentType);
private:
	class CMIMETypeManager
	{
	public:
		CMIMEContentAgent* GetHandler( int nContentType );
		void RegisterMIMEType( CMIMEContentAgent* pMIMEType);
		virtual  ~CMIMETypeManager();
		CMIMETypeManager();
	private:
		CCriticalSection m_csAccess;
		CList < CMIMEContentAgent*, CMIMEContentAgent* > m_MIMETypeList;
	};
	CMIMEPart* m_pMainMIMEPart;
	public:
		bool m_FirstMultipart;
	static CMIMETypeManager m_MIMETypeManager;
};


#endif // !defined(AFX_MIMEMESSAGE_H__6233D556_EBBF_11D1_870E_444553540001__INCLUDED_)
