// MailMessage.cpp: implementation of the CMailMessage class.
// Copyright (c) 1998, Wes Clyburn
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include"longstr.h"
#include "MailMessage.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CMailMessage::CMailMessage()
{
	m_sMailerName = _T( "TOL Mailer" );
	SetCharsPerLine( 76 );
}

CMailMessage::~CMailMessage()
{

}

BOOL CMailMessage::AddRecipient( LPCTSTR szEmailAddress, LPCTSTR szFriendlyName, typeRecipient Type)
{
	ASSERT( szEmailAddress != NULL );
	ASSERT( szFriendlyName != NULL );
	CRecipient to;
	to.m_sEmailAddress = szEmailAddress;
	to.m_sFriendlyName = szFriendlyName;
	to.m_Type = Type;
	m_Recipients.Add( to );
	return TRUE;
}

// sEmailAddress and sFriendlyName are OUTPUT parameters.
// If the function fails, it will return FALSE, and the OUTPUT
// parameters will not be touched.
BOOL CMailMessage::GetRecipient(CString & sEmailAddress, CString & sFriendlyName, typeRecipient & Type, int nIndex)
{
	CRecipient to;
	if( nIndex < 0 || nIndex > m_Recipients.GetUpperBound() )
		return FALSE;
	to = m_Recipients[ nIndex ];
	sEmailAddress = to.m_sEmailAddress;
	sFriendlyName = to.m_sFriendlyName;
	Type = to.m_Type;
	return TRUE;
}

int CMailMessage::GetNumRecipients()
{
	return m_Recipients.GetSize();
}

BOOL CMailMessage::AddMultipleRecipients(LPCTSTR szRecipients, typeRecipient Type)
{
	TCHAR* buf;
	UINT pos;
	UINT start;
	CString sTemp;
	CString sEmail;
	CString sFriendly;
	UINT length;
	int nMark;
	int nMark2;

	ASSERT( szRecipients != NULL );
	
	// Add Recipients
	//
	length = strlen( szRecipients );
	buf = new TCHAR[ length + 1 ];	// Allocate a work area (don't touch parameter itself)
	strcpy( buf, szRecipients );
	for( pos = 0, start = 0; pos <= length; pos++ )
	{
		if( buf[ pos ] == ';' ||
			buf[ pos ] == 0 )
		{
			// First, pick apart the sub-strings (separated by ';')
			//  Store it in sTemp.
			//
			buf[ pos ] = 0;	// Redundant when at the end of string, but who cares.
			sTemp = &buf[ start ];

			// Now divide the substring into friendly names and e-mail addresses.
			//
			nMark = sTemp.Find( '<' );
			if( nMark >= 0 )
			{
				sFriendly = sTemp.Left( nMark );
				nMark2 = sTemp.Find( '>' );
				if( nMark2 < nMark )
				{
					delete[] buf;
					return FALSE;
				}
				// End of mark at closing bracket or end of string
				nMark2 > -1 ? nMark2 = nMark2 : nMark2 = sTemp.GetLength() - 1;
				sEmail = sTemp.Mid( nMark + 1, nMark2 - (nMark + 1) );
			}
			else
			{
				sEmail = sTemp;
				sFriendly = _T( "" );
			}
			AddRecipient( sEmail, sFriendly, Type );
			start = pos + 1;
		}
	}
	delete[] buf;
	return TRUE;
}

void CMailMessage::FormatMessage()
{
	NotifyProgress(0,NULL);
	start_header();
	prepare_header();
	end_header();
	prepare_body();
	NotifyProgress(2,NULL);
}

void CMailMessage::SetCharsPerLine(UINT nCharsPerLine)
{
	m_nCharsPerLine = nCharsPerLine;
}	

UINT CMailMessage::GetCharsPerLine()
{
	return m_nCharsPerLine;
}

// Create header as per RFC 822
//
void CMailMessage::prepare_header()
{
	CString sTemp;

	sTemp = _T( "" );
	// From:
	sTemp = _T( "From: " ) + m_sSender + m_sFrom;
	add_header_line( (LPCTSTR)sTemp );
	
// PC 2001/3
	CString To,Cc;
	CString sEmail;
	CString sFriendly;
	typeRecipient Type;
	for( int i = 0; i < GetNumRecipients(); i++ )
	{
		GetRecipient( sEmail, sFriendly, Type, i );
		sTemp = sFriendly;
		sTemp += _T( "<" );
		sTemp += sEmail;
		sTemp += _T( ">" );
		if(Type == TO) 
			if(To.IsEmpty())
				To = "To: "+sTemp;
			else
				To += sTemp;
		else if(Type == CC) 
			if(Cc.IsEmpty())
				Cc = "Cc: "+sTemp;
			else
				Cc += sTemp;
	}
	sTemp = _T( "To: " );
	if(!To.IsEmpty()) add_header_line( (LPCTSTR) To );
	if(!Cc.IsEmpty()) add_header_line( (LPCTSTR) Cc );
	{
		// Date:
		m_tDateTime = m_tDateTime.GetCurrentTime();
		// Format: Mon, 01 Jun 98 01:10:30 GMT
		sTemp = _T( "Date: " );
//		sTemp += m_tDateTime.Format( "%a, %d %b %y %H:%M:%S %Z" ); PC 21/9/01
		sTemp += m_tDateTime.Format( "%a, %d %b %Y %H:%M:%S ");


		TIME_ZONE_INFORMATION timezoneinfo;
		LONG lTimeDifferential = 0;
		LONG lHours = 0;
		LONG lMinutes = 0;
		DWORD dwReturn  = GetTimeZoneInformation(&timezoneinfo);
		switch (dwReturn)
		{
		case TIME_ZONE_ID_STANDARD:
			lTimeDifferential = timezoneinfo.Bias + timezoneinfo.StandardBias;
			//Bias is in minutes--calculate the hours for HHMM format.
			lHours = -(lTimeDifferential/60);
			//Bias is in minutes--calculate the minutes for HHMM format.
			lMinutes = lTimeDifferential%60L;
			break;
		case TIME_ZONE_ID_DAYLIGHT:
			lTimeDifferential = timezoneinfo.Bias + timezoneinfo.DaylightBias;
			//Bias is in minutes--calculate the hours for HHMM format.
			//Need to take the additive inverse.
			//Bias is based on GMT=Local+Bias.
			//We need a differential based on GMT=Local-Bias.
			lHours = -(lTimeDifferential/60);
			//Bias is in minutes--calculate the minutes for HHMM format.
			lMinutes = lTimeDifferential%60L;
			break;
		case TIME_ZONE_ID_INVALID:
			ASSERT(FALSE);
			break;
		}
		CString Offset;
		Offset.Format(_T("%+.2d%.2d"), lHours, lMinutes);

		sTemp += Offset;
		add_header_line( (LPCTSTR)sTemp );
	}

	// Subject:
	sTemp = _T( "Subject: " ) + m_sSubject;
	add_header_line( (LPCTSTR)sTemp );

	// X-Mailer
	sTemp = _T( "X-Mailer: " ) + m_sMailerName;
	add_header_line( (LPCTSTR)sTemp );
}

void CMailMessage::prepare_body()
{
	// Append a CR/LF to body if necessary.
	if( m_sBody.Right( 2 ) != _T( "\r\n" ) )
		m_sBody += _T( "\r\n" );

}


void CMailMessage::start_header()
{
	m_sHeader = _T( "" );
}

void CMailMessage::end_header()
{
	m_sHeader += _T( "\r\n" );
}

void CMailMessage::add_header_line(LPCTSTR szHeaderLine)
{
	CString sTemp;
	sTemp.Format( _T( "%s\r\n" ), szHeaderLine );
	m_sHeader += sTemp;
}



void CMailMessage::NotifyProgress(int Encoding,LPCSTR Filnam)
{
	if(Filnam != NULL)
		TRACE("\nNotifyProgress : file <%s> :",Filnam);
	else
		TRACE("<%dk>",Encoding);
}
