// SMTP.cpp: implementation of the CSMTP class.
// Copyright (c) 1998, Wes Clyburn
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include"longstr.h"
#include "SMTP.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

// Static member initializers
//

// Note: the order of the entries is important.
//       They must be synchronized with eResponse entries. 
CSMTP::response_code CSMTP::response_table[] =
{
	// GENERIC_SUCCESS
	{ 250, _T( "SMTP server error" ) },

	// CONNECT_SUCCESS
	{ 220, _T( "SMTP server not available" ) },

	// DATA_SUCCESS
	{ 354, _T( "SMTP server not ready for data" ) },

	// QUIT_SUCCESS
	{ 221, _T( "SMTP server didn't terminate session" ) }
};


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSMTP::CSMTP( LPCTSTR szSMTPServerName, UINT nPort )
{
	AfxSocketInit();
	m_sSMTPServerHostName = szSMTPServerName;
	m_nPort = nPort;
	m_bConnected = FALSE;
	m_sError = _T( "OK" );
	response_buf = NULL;
}

CSMTP::~CSMTP()
{
	Disconnect();
}

CString CSMTP::GetServerHostName()
{
	return m_sSMTPServerHostName;
}

BOOL CSMTP::Connect()
{
	ASSERT( !m_sSMTPServerHostName.IsEmpty());
	CString sHello;
	TCHAR local_host[ 80 ];	// Warning: arbitrary size
	if( m_bConnected )
		return TRUE;
	
	try
	{
		// This will be deleted in Disconnect();
		response_buf = new TCHAR[ RESPONSE_BUFFER_SIZE ];

		// I can't count on all class users' applications
		// to have exception-throwing operator-new implementations,
		// so I'll soul-kiss the ones that don't.
		if( response_buf == NULL )
		{
			m_sError = _T( "Not enough memory" );
			return FALSE;
		}
	}
	catch( CException *e )
	{
		response_buf = NULL;
		m_sError = _T( "Not enough memory" );
		delete e;
		return FALSE;
	}

	if( !m_wsSMTPServer.Create() )
	{
		m_sError = _T( "Unable to create the socket." );
		delete response_buf;
		response_buf = NULL;
		return FALSE;
	}
	if( !m_wsSMTPServer.Connect( GetServerHostName(), GetPort() ) )
	{
		m_sError = _T( "Unable to connect to server" );
		m_wsSMTPServer.Close();
		delete response_buf;
		response_buf = NULL;
		return FALSE;
	}
	if( !get_response( CONNECT_SUCCESS ) )
	{
		m_sError = _T( "Server didn't respond." );
		m_wsSMTPServer.Close();
		delete response_buf;
		response_buf = NULL;
		return FALSE;
	}
	gethostname( local_host, 80 );
	sHello.Format( _T( "HELO %s\r\n" ), local_host );
	m_wsSMTPServer.Send( (LPCTSTR)sHello, sHello.GetLength() );
	if( !get_response( GENERIC_SUCCESS ) )
	{
		m_wsSMTPServer.Close();
		delete response_buf;
		response_buf = NULL;
		return FALSE;
	}
	m_bConnected = TRUE;
	return TRUE;
}

BOOL CSMTP::Disconnect()
{
	BOOL ret;
	if( !m_bConnected )
		return TRUE;
	// Disconnect gracefully from the server and close the socket
	CString sQuit = _T( "QUIT\r\n" );
	m_wsSMTPServer.Send( (LPCTSTR)sQuit, sQuit.GetLength() );

	// No need to check return value here.
	// If it fails, the message is available with GetLastError
	ret = get_response( QUIT_SUCCESS );
	m_wsSMTPServer.Close();

	if( response_buf != NULL )
	{
		delete[] response_buf;
		response_buf = NULL;
	}

	m_bConnected = FALSE;
	return ret;
}

UINT CSMTP::GetPort()
{
	return m_nPort;
}

CString CSMTP::GetLastError()
{
	return m_sError;
}

BOOL CSMTP::SendMessage(CMailMessage * msg,bool SoonFormated)
{
	ASSERT( msg != NULL );
	if( !m_bConnected )
	{
		m_sError = _T( "Must be connected" );
		return FALSE;
	}
	if(!SoonFormated)
	{
		if( FormatMailMessage( msg ) == FALSE )
		{
			return FALSE;
		}
	}
	if( transmit_message( msg ) == FALSE )
	{
		return FALSE;
	}
	return TRUE;
}

BOOL CSMTP::FormatMailMessage( CMailMessage* msg )
{
	ASSERT( msg != NULL );
/*	if( msg->GetNumRecipients() == 0 ) // PC 3/2001 allow  no "To:"
	{
		m_sError = _T( "No Recipients" );
		return FALSE;
	}
*/
	msg->FormatMessage();
	return TRUE;
}

void CSMTP::SetServerProperties( LPCTSTR szSMTPServerName, UINT nPort)
{
	ASSERT( szSMTPServerName != NULL );
	// Needs to be safe in non-debug too
	if( szSMTPServerName == NULL )
		return;
	m_sSMTPServerHostName = szSMTPServerName;
	m_nPort = nPort;
}



CString CSMTP::cook_body(CMailMessage * msg)
{
	ASSERT( msg != NULL );
	CLongString sTemp;
	int Len=msg->m_sBody.GetLength();
	LPCSTR pIn=msg->m_sBody.GetBuffer(Len);
	sTemp.OptimizeGrowing(300000);
	int Indx=0;
	char c=0;
	char Oldc;
	if(memcmp(pIn,".\r\n",3)==0)
	{
		sTemp += ".";
		Indx++;
	}
	
	while(Indx<Len)
	{
		Oldc=c;
		c=pIn[Indx++];
		if(c=='\n')
		{
			if(Oldc!='\r')
			{
				sTemp += '\r';
			}		
			if(pIn[Indx]=='.')
			{
				if(Indx<Len-1)
				{
					if( (pIn[Indx+1]=='\r') || (pIn[Indx+1]=='\n') )
					{
						sTemp += "\n.";
						Indx++;
						c='.';
					}

				}
			}
		}
		sTemp += c;
	}

	sTemp.EndOptimizeGrowing();

	return sTemp;
}



/*
CString CSMTP::cook_body(CMailMessage * msg)
{
	ASSERT( msg != NULL );
	CString sTemp;
	CString sCooked = _T( "" );
	LPTSTR szBad = _T( "\r\n.\r\n" );
	LPTSTR szGood = _T( "\r\n..\r\n" );
	int nPos;
	int nStart = 0;
	int nBadLength = strlen( szBad );
	sTemp = msg->m_sBody;
	if( sTemp.Left( 3 ) == _T( ".\r\n" ) )
		sTemp = _T( "." ) + sTemp;
	//
	// This is a little inefficient because it beings a search
	// at the beginning of the string each time. This was
	// the only thing I could think of that handled ALL variations.
	// In particular, the sequence "\r\n.\r\n.\r\n" is troublesome. 
	// (Even CStringEx's FindReplace wouldn't handle that situation
	// with the global flag set.)
	//
	while( (nPos = sTemp.Find( szBad )) > -1 )
	{
		sCooked = sTemp.Mid( nStart, nPos );
		sCooked += szGood;
		sTemp = sCooked + sTemp.Right( sTemp.GetLength() - (nPos + nBadLength) );
	}
	return sTemp;
}
*/

BOOL CSMTP::transmit_message(CMailMessage * msg)
{
	CString sFrom;
	CString sTo;
	CString sTemp;
	CString sEmail;

	ASSERT( msg != NULL );
	if( !m_bConnected )
	{
		m_sError = _T( "Must be connected" );
		return FALSE;
	}

	// Send the MAIL command
	//
	sFrom.Format( _T( "MAIL From: %s\r\n" ), (LPCTSTR)msg->m_sFrom );
	m_wsSMTPServer.Send( (LPCTSTR)sFrom, sFrom.GetLength() );
	if( !get_response( GENERIC_SUCCESS ) )
		return FALSE;
	
	// Send RCPT commands (one for each recipient)
	//
	CMailMessage::typeRecipient Type;
	for( int i = 0; i < msg->GetNumRecipients(); i++ )
	{
		msg->GetRecipient( sEmail, sTemp, Type, i );
		sTo.Format( _T( "RCPT TO: <%s>\r\n" ), (LPCTSTR)sEmail );
		m_wsSMTPServer.Send( (LPCTSTR)sTo, sTo.GetLength() );
		get_response( GENERIC_SUCCESS );
	}

	// Send the DATA command
	sTemp = _T( "DATA\r\n" );
	m_wsSMTPServer.Send( (LPCTSTR)sTemp, sTemp.GetLength() );
	if( !get_response( DATA_SUCCESS ) )
	{
		return FALSE;
	}
	// Send the header
	//
	m_wsSMTPServer.Send( (LPCTSTR)msg->m_sHeader, msg->m_sHeader.GetLength() );

	// Send the body
	//

	sTemp = cook_body( msg );
	m_wsSMTPServer.Send( (LPCTSTR)sTemp, sTemp.GetLength() );

	// Signal end of data
	//
	sTemp = _T( "\r\n.\r\n" );
	m_wsSMTPServer.Send( (LPCTSTR)sTemp, sTemp.GetLength() );
	if( !get_response( GENERIC_SUCCESS ) )
	{
		return FALSE;
	}
	return TRUE;
}

BOOL CSMTP::get_response( UINT response_expected )
{
	ASSERT( response_expected >= GENERIC_SUCCESS );
	ASSERT( response_expected < LAST_RESPONSE );

	CString sResponse;
	UINT response;
	response_code* pResp;	// Shorthand

	if( m_wsSMTPServer.Receive( response_buf, RESPONSE_BUFFER_SIZE ) == SOCKET_ERROR )
	{
		m_sError = _T( "Socket Error" );
		return FALSE;
	}
	sResponse = response_buf;

#ifdef _DEBUGHERE
		CString STrace(sResponse);
		int Trlen=STrace.GetLength();
		if(Trlen>30) STrace.GetBufferSetLength(30);
		TRACE("############ %s\n",STrace);
#endif
	sscanf( (LPCTSTR)sResponse.Left( 3 ), _T( "%d" ), &response );
	pResp = &response_table[ response_expected ];
	if( response != pResp->nResponse )
	{
		m_sError.Format( _T( "%d:%s" ), response, (LPCTSTR)pResp->sMessage );
		return FALSE;
	}
	return TRUE;
}

/*
Return-Path: <pecouderc@tol.fr<Pierre Couderc>
Delivered-To: pecouderc@tol.fr
Received: (qmail 3563 invoked by alias); 18 Apr 2000 14:11:52 -0000
Received: from unknown (HELO TOLL) (10.0.0.15)
  by 10.0.0.8 with SMTP; 18 Apr 2000 14:11:52 -0000
From: pecouderc@tol.fr<Pierre Couderc>
To: <pecouderc@tol.fr>
Date: Tue, 18 Apr 00 16:07:34 Paris, Madrid (heure d't)
Subject: TOLSenditV1_Msg 01/01
X-Mailer: WC Mail
MIME-Version: 1.0
Content-Type: multipart/mixed; boundary=WC_MAIL_PaRt_BoUnDaRy_05151998

This is a multi-part message in MIME format.

Return-Path: <pcouderc@tol.fr>
Delivered-To: pecouderc@tol.fr
Received: (qmail 3878 invoked by alias); 18 Apr 2000 16:11:07 -0000
Received: from unknown (HELO TOLL) (10.0.0.15)
  by 10.0.0.8 with SMTP; 18 Apr 2000 16:11:07 -0000
Message-ID: <007601bfa950$19fc5bb0$0f00000a@TOLL>
From: "Pierre Couderc" <pcouderc@tol.fr>
To: <pecouderc@tol.fr>
Subject: ggg
Date: Tue, 18 Apr 2000 18:06:48 +0200


Return-Path: <bidon2bidon3pecouderc@tol.fr<Moi meme>
Delivered-To: pecouderc@tol.fr
Received: (qmail 4204 invoked by alias); 18 Apr 2000 18:21:35 -0000
Received: from unknown (HELO TOLL) (10.0.0.15)
  by 10.0.0.8 with SMTP; 18 Apr 2000 18:21:35 -0000
From: bidon1bidon3pecouderc@tol.fr<Moi meme>
To: <pecouderc@tol.fr>
Date: Tue, 18 Apr 00 20:17:16 Paris, Madrid (heure d't)
Subject: TOLSenditV1_Msg 01/01
X-Mailer: WC Mail


*/

