// TextPlain.cpp: implementation of the CTextPlain class.
// Author: Wes Clyburn (clyburnw@enmu.edu)
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "longstr.h"
#include "TextPlain.h"
#include "MIMEMessage.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CTextPlain::CTextPlain( int nContentType, UINT nWrapPos )
	:CMIMEContentAgent( nContentType)
{
	m_nWrapPos = nWrapPos;
	m_nContentType = nContentType;
}

CTextPlain::~CTextPlain()
{

}

CString CTextPlain::GetContentTypeString()
{
	CString s;
	if( m_nContentType == CMIMEMessage::TEXT_HTML)
		s = _T( "text/html;\r\n\tcharset=\"iso-8859-1\"" );
	else
		s = _T( "text/plain" );
	return s;
}

BOOL CTextPlain::AppendPart(LPCTSTR szContent, 
							LPCTSTR szParameters, 
							int nEncoding, 
							CLongString & sDestination,
							CMIMEMessage *pMIMEMessage )
{
	CString sSubHeader;
	CString sWrapped;
	m_pMIMEMessage=pMIMEMessage;
	sSubHeader = build_sub_header( szContent,
								   szParameters,
								   nEncoding);
	sWrapped = wrap_text( szContent );
	sDestination += (sSubHeader + sWrapped);
	return TRUE;
}

CString CTextPlain::build_sub_header(LPCTSTR szContent, 
									 LPCTSTR szParameters, 
									 int nEncoding)
{
	CString sSubHeader;
	sSubHeader.Format( _T( "Content-Type: %s%s\r\n" ),
					   (LPCTSTR)GetContentTypeString(),
					   szParameters );
	sSubHeader += _T( "Content-Transfer-Encoding: " );

	ASSERT( ((nEncoding & CMIMEMessage::ENCODE)==0) || ( !(nEncoding & CMIMEMessage::QUOTED_PRINTABLE)!=0) ); 
//         if you fall in this assert, you have tried to encode in quoted-printable : this is not soon implemented
//         This class handles DOES NO ENCODING, if you decide to implement it,
//				see http://www.codeproject.com/string/ammimeutils.asp
//				and send me the result at pcouderc@tol.fr
	ASSERT( ((nEncoding & CMIMEMessage::ENCODE)==0) || ( !(nEncoding & CMIMEMessage::_7BIT)!=0) ); 
//         if you fall in this assert, you have tried to encode in _7BIT
//         This class handles DOES NO ENCODING
	int Encoding = nEncoding & 0x0ffff;
	if( Encoding == CMIMEMessage::DEFAULT)
		switch( m_nContentType )
		{
			case CMIMEMessage::TEXT_HTML:
				Encoding= CMIMEMessage::QUOTED_PRINTABLE ;
				break;
			case CMIMEMessage::TEXT_PLAIN:
				Encoding= CMIMEMessage::_7BIT;
				break;			
		}
	switch( Encoding)
	{
		default:
			ASSERT(FALSE); // Format not supported
			//Fall through to...

		case CMIMEMessage::_7BIT:
			sSubHeader += _T( "7Bit" );
			break;
		case CMIMEMessage::QUOTED_PRINTABLE:
			sSubHeader += _T( "quoted-printable" );
			break;			
	}
	sSubHeader += _T( "\r\n\r\n" );
	return sSubHeader;
}

CString CTextPlain::wrap_text(LPCTSTR szText)
{
	CString sTemp;
	CString sLeft;
	CString sRight;
	int lp = 0;
	UINT nCount = 0;
	int nSpacePos = 0;

	ASSERT( szText != NULL );
	if( szText == NULL )
		return sTemp;
	sTemp = szText;
	while( lp < sTemp.GetLength() )
	{
#ifdef _DEBUG
//           an ASSERT here detects a '\n' without '\r'
				if(sTemp[ lp ] == '\n') ASSERT(sTemp[ lp-1 ] == '\r');
#endif
		if( sTemp[ lp ] == ' ' )
			nSpacePos = lp;
		// Reset counter on newline
		if( sTemp.Mid( lp, 2 ) == _T( "\r\n" ) )
			nCount = 0;
		// Wrap text at last found space
		if( nCount > m_nWrapPos )
		{
			sLeft = sTemp.Left( nSpacePos );
			sRight = sTemp.Right( sTemp.GetLength() - nSpacePos );
			sLeft.TrimRight();
			sRight.TrimLeft();
			sLeft += _T( "\r\n" );
			sTemp = sLeft + sRight;
			nCount = 0;
		}
		else
			nCount++;
		lp++;
	}
	return sTemp;
}
